package hok.model;

import hok.ui.wizard.IConstraintViewer;
import hok.ui.wizard.WizardView;

import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

/**
 * The ConstraintModel is the model of the TableViewer of the {@link WizardView}. It contains
 * the rows (ConstraintRow) of the table and the listeners that wish to be
 * informed of changes in the model.
 * 
 * @author holger
 * 
 */
public class ConstraintModel {
	private final int COUNT = 10;

	private Vector<ConstraintRow> constraints = new Vector<ConstraintRow>(
			COUNT);

	private Set<IConstraintViewer> changeListeners = new HashSet<IConstraintViewer>();

	/**
	 * The default value of the concept constraint part of a
	 * ConstraintRow.
	 */
	public static final String INIT_VALUE_CONCEPT = "<concept constraint>";

	/**
	 * The default value of the predicate constraint part of a
	 * ConstraintRow.
	 */
	public static final String INIT_VALUE_PREDICATE = "<predicate constraint>";

	/**
	 * Returns the collection of constraints
	 * 
	 * @return the vector of SparqlConstraintRows
	 */
	public Vector<ConstraintRow> getConstraints() {
		return constraints;
	}

	/**
	 * Adds a new ConstraintRow to the collection of constraints. It also
	 * informs the listeners of the change of the model.
	 */
	public void addConstraint() {
		ConstraintRow constraint = new ConstraintRow();
		constraint.setConceptConstraint(INIT_VALUE_CONCEPT);
		constraint.setPredicateConstraint(INIT_VALUE_PREDICATE);
		constraints.add(constraints.size(), constraint);

		for (IConstraintViewer viewer : changeListeners) {
			viewer.addConstraint(constraint);
		}
	}

	/**
	 * Removes a ConstraintRow from the collection of constraints. It also
	 * informs the listeners of the change of the model.
	 * 
	 * @param constraint
	 *            the SparqlConstraintrow to remove.
	 */
	public void removeConstraint(ConstraintRow constraint) {

		for (IConstraintViewer viewer : changeListeners) {
			viewer.removeConstraint(constraint);
		}
		constraints.remove(constraint);
	}

	/**
	 * Informs the listener of a change of the model.
	 * 
	 * @param constraint
	 *            the ConstraintRow that has changed.
	 */
	public void constraintChanged(ConstraintRow constraint) {

		for (IConstraintViewer viewer : changeListeners) {
			viewer.updateConstraint(constraint);
		}
	}

	/**
	 * Removes a listener from the model.
	 * 
	 * @param viewer
	 *            the listener to remove.
	 */
	public void removeChangeListener(IConstraintViewer viewer) {
		changeListeners.remove(viewer);
	}

	/**
	 * Adds a listener to the model.
	 * 
	 * @param viewer
	 *            the listner to add.
	 */
	public void addChangeListener(IConstraintViewer viewer) {
		changeListeners.add(viewer);
	}

	/**
	 * Checks if the model contains elements.
	 * 
	 * @return true if the model is empty, false otherwise.
	 */
	public boolean isEmpty() {
		return constraints.isEmpty();
	}

}
