package hok.model;

import java.io.InputStream;
import java.util.ResourceBundle;
import java.util.Vector;

import org.antlr.stringtemplate.StringTemplate;
import org.antlr.stringtemplate.StringTemplateGroup;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.reasoner.ReasonerRegistry;
import com.hp.hpl.jena.reasoner.dig.DIGReasoner;
import com.hp.hpl.jena.reasoner.dig.DIGReasonerFactory;
import com.hp.hpl.jena.vocabulary.ReasonerVocabulary;

/**
 * The {@link ModelMediator} mediates between the ontology model and the sparql
 * ast model. It provides methods to generate the AST from Sparql constraint
 * rows which are configured by the user in the view and to use the ast to query
 * the ontology.
 * 
 * @author holger
 * @uml.dependency supplier="hok.model.PatternModel"
 * @uml.dependency supplier="hok.model.TreeFactory" stereotypes="Meta::use"
 * @uml.dependency supplier="hok.model.ConstraintModel"
 */
public class ModelMediator {

	private String ontPath = "/dpIntentOntology_v0.5.4.owl";

	private static final String TEMPLATE_URL = "tpl/sparql";

	private static final String ONT_BASE_URI = "http://www.holger-kampffmeyer.de/DesignpatternsIntentOntology.owl#";

	private String digUrl = "http://localhost:8081";

	public static final String RESOURCE_PATH = "config";

	public static final String REASONER_URL = "reasoner.url";

	public static final String ONT_PATH = "ontology.path";

	private ITreeNode conceptTree;

	private ITreeNode predicateTree;

	private OntModel m;

	private Log log = LogFactory.getLog(ModelMediator.class);

	private SparqlContext context;

	private StringTemplateGroup group;

	ResourceBundle config;

	/**
	 * Constructor. Calls the init methods to initialize the OWL and the Sparql
	 * model.
	 * 
	 */
	public ModelMediator() {

		config = ResourceBundle.getBundle(RESOURCE_PATH);
		if (config == null) {
			log.error("Config file " + RESOURCE_PATH + " not found!");
		}

		initReasoner();
		initQueryTemplateEngine();
		initOntModel();
	}

	private void initReasoner() {

		if (!config.containsKey(REASONER_URL)) {
			log
					.error("Error in config.properties. The URL of the reasoner is not set!");
		}
		this.digUrl = config.getString(REASONER_URL);

	}

	/**
	 * Initializes the OWL Model.
	 */
	private void initOntModel() {
		
		if (!config.containsKey(ONT_PATH)) {
			log.error("Error in config.properties. The path to the ontology is not set!");
		}
		this.ontPath = config.getString(ONT_PATH);
		try {

			/*
			 * Dig reasoner with base and standard model
			 */
			OntModel base = ModelFactory.createOntologyModel(
					OntModelSpec.OWL_DL_MEM, null);
			InputStream ontStream = getClass()
					.getResourceAsStream(this.ontPath);
			if (ontStream == null) {

				throw new OntElementNotFoundException("The ontology file "
						+ this.ontPath + " could not be found!");
			}
			base.read(ontStream, null);
			log.info("Ontology "+this.ontPath+" loaded.");

			OntModelSpec spec = new OntModelSpec(OntModelSpec.OWL_DL_MEM);
			Resource conf = base.createResource();
			conf.addProperty(ReasonerVocabulary.EXT_REASONER_URL, base
					.createResource(digUrl));
			// create the reasoner factory and the reasoner
			DIGReasonerFactory drf = (DIGReasonerFactory) ReasonerRegistry
					.theRegistry().getFactory(DIGReasonerFactory.URI);
			DIGReasoner r = (DIGReasoner) drf.create(conf);

			if (r == null) {
				throw new OntElementNotFoundException("No DIG reasoner found!");
			}

			spec.setReasoner(r);

			m = ModelFactory.createOntologyModel(spec, base);

			TreeFactory treeFactory = new TreeFactory(m);
			this.conceptTree = treeFactory.generateConceptTree(ONT_BASE_URI
					+ TreeFactory.ROOT_CONCEPT);
			this.predicateTree = treeFactory.generatePredicateTree(ONT_BASE_URI
					+ TreeFactory.ROOT_PREDICATE);
		} catch (OntElementNotFoundException e) {
			log.fatal("Initialization of model failed!");
			log.fatal(e);
		}

	}

	private void initQueryTemplateEngine() {
		// initialize the template engine for generating
		// the SPARQL query.
		this.group = new StringTemplateGroup("s", (String) null);
		// this.sparqltemplate = this.group.getInstanceOf("tpl/sparql");
		this.context = new SparqlContext();
	}

	/**
	 * Returns the concept tree.
	 * 
	 * @return the concept tree.
	 */
	public ITreeNode getConceptTree() {
		return conceptTree;
	}

	/**
	 * Returns the predicate tree.
	 * 
	 * @return the predicate tree.
	 */
	public ITreeNode getPredicateTree() {
		return predicateTree;
	}

	/**
	 * Creates the ontology query from a collection of {@link ConstraintRow}.
	 * 
	 * @param constraints
	 *            the collection of {@link ConstraintRow} from which to create
	 *            the query.
	 * @return the String representation of the query.
	 * @throws QueryCreationFailedException
	 *             if the query template could not be found.
	 */
	private String createOntQuery(Vector<ConstraintRow> constraints)
			throws QueryCreationFailedException {

		StringTemplate sparqltemplate = group.getInstanceOf(TEMPLATE_URL);
		if (sparqltemplate == null) {
			throw new QueryCreationFailedException(
					"Could not create the query from the query template!");
		}

		for (ConstraintRow row : constraints) {
			if (!row.getPredicateConstraint().equals(
					ConstraintModel.INIT_VALUE_PREDICATE)) {
                String constrVar = this.context.generateVar();
                String problVar = this.context.generateVar();

                sparqltemplate.setAttribute("objectvar", constrVar);
                sparqltemplate.setAttribute ("problemvar", problVar);
				sparqltemplate.setAttribute("predicate", row
						.getPredicateConstraint());
				sparqltemplate.setAttribute("object", row
						.getConceptConstraint().equals(
								ConstraintModel.INIT_VALUE_CONCEPT) ? null
						: row.getConceptConstraint());
			}
		}
		log.info(sparqltemplate.toString());
		return sparqltemplate.toString();
	}

	/**
	 * Queries the ontology model.
	 * 
	 * @param constraints
	 *            the collection of {@link ConstraintRow} from which to create
	 *            the query.
	 * @return the pattern model containing the query result set.
	 * @throws QueryCreationFailedException
	 *             is thrown if the query creatio has failed.
	 * @throws OntElementNotFoundException
	 *             is thrown if an ontology element is not found.
	 */
	public PatternModel queryModel(Vector<ConstraintRow> constraints)
			throws QueryCreationFailedException, OntElementNotFoundException {

		log.info("Execute Query.");
		String query = createOntQuery(constraints);
		if (query == null || query.equals("")) {

			throw new QueryCreationFailedException(
					"The creation of the Sparql query failed!");
		}

		PatternModel pModel = new PatternModel();
		if (this.m == null) {
			throw new OntElementNotFoundException(
					"Ontology model has not been initialized!");
		}
		QueryExecution qexec = QueryExecutionFactory.create(query, this.m);
		try {
			ResultSet results = qexec.execSelect();
			while (results.hasNext()) {
				QuerySolution soln = results.nextSolution();

				PatternElement pElem = new PatternElement();

				RDFNode node = soln.get(SparqlContext.SPARQL_DP_VAR);
				if (node.canAs(Individual.class)) {
					Individual cls = (Individual) node.as(Individual.class);

					pElem.setDescription(cls.getComment("EN"));
				}

				node = soln.get(SparqlContext.SPARQL_DPNAME_VAR);
				if (node.canAs(Literal.class)) {
					Literal prop = (Literal) node.as(Literal.class);
					pElem.setName(prop.getLexicalForm());
				}
				pModel.addPattern(pElem);
			}

		} catch (Exception e) {
			log.fatal(e);
			e.printStackTrace();
		} finally {
			log.info("Closing query.");
			qexec.close();
		}

		return pModel;
	}

	class SparqlContext {

		static final String SPARQL_DP_VAR = "?dp";

		static final String SPARQL_DPNAME_VAR = "?name";

		private int count = 0;

		public String generateVar() {
			return "?var" + (this.count++);
		}
	}

}
