package hok.model;

import hok.ui.wizard.ConstraintDialog;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.TreeViewer;

import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/**
 * The {@link TreeFactory} provides methods to generate a composite of
 * {@link ITreeNode} . The generated tree is the model used by the
 * {@link TreeViewer} of the {@link ConstraintDialog} to represent a tree of owl
 * model instances.
 * 
 * @author holger
 * @uml.dependency supplier="hok.model.ITreeNode" stereotypes="Standard::Create"
 */
public class TreeFactory {

	/**
	 * The root concept of the owl vocabulary to describe a design problem.
	 */
	public static final String ROOT_CONCEPT = "ProblemConcept";

	/**
	 * The root predicate of the owl vocabulary to describe a design problem.
	 */
	public static final String ROOT_PREDICATE = "constrains";

	private OntModel ontmodel;

	/**
	 * The Constructor. It stores a Jena {@link OntModel}.
	 * 
	 * @param ontmodel
	 *            the owl model to store.
	 * @throws OntElementNotFoundException
	 *             is thrown when the ontmodel is null.
	 */
	public TreeFactory(OntModel ontmodel) throws OntElementNotFoundException {
		if (ontmodel == null) {
			throw new OntElementNotFoundException(
					"The Ontology Model must not be null!");
		}
		this.ontmodel = ontmodel;
	}

	/**
	 * Generates the concept tree of the owl model starting with the given root
	 * concept.
	 * 
	 * @param rootConcept
	 *            the owl class from which to start constructing the tree.
	 * @return a composite tree representation of {@link ITreeNode}.
	 * @throws OntElementNotFoundException
	 *             is thrown when the rootConcept does not exist in the
	 *             ontology.
	 */
	public ITreeNode generateConceptTree(String rootConcept)
			throws OntElementNotFoundException {
		OntClass ontclass = this.ontmodel.getOntClass(rootConcept);
		if (ontclass == null) {
			throw new OntElementNotFoundException("The root concept "
					+ rootConcept + " does not exist!");
		}
		ITreeNode rootElement = populateElement(ontclass, null);
		generateChildConcepts(ontclass, rootElement, new ArrayList<OntClass>(),
				0);
		return rootElement;
	}

	private void generateChildConcepts(OntClass cls, ITreeNode parentModel,
			List<OntClass> occurs, int depth) {

		ITreeNode curElement = populateElement(cls, parentModel);
		parentModel.addChild(curElement);

		// recurse to the next level down
		if (cls.canAs(OntClass.class) && !occurs.contains(cls)) {
			for (ExtendedIterator it = cls.listSubClasses(true); it.hasNext();) {
				OntClass sub = (OntClass) it.next();

				if (sub.isOntLanguageTerm()) {
					continue;
				}
				// we push this expression on the occurs list before we recurse
				occurs.add(cls);
				generateChildConcepts(sub, curElement, occurs, depth + 1);
				occurs.remove(cls);
			}
		}
	}

	private ITreeNode populateElement(OntClass cls, ITreeNode parent) {
		TreeElement modelElement = new TreeElement(parent);
		modelElement.setLabel(cls.getLabel(null));
		modelElement.setComment(cls.getComment(null));
		modelElement.setElement(cls.getLocalName());
		return modelElement;
	}

	private ITreeNode populateElement(OntProperty opr, ITreeNode parent) {
		TreeElement modelElement = new TreeElement(parent);
		modelElement.setLabel(opr.getLabel(null));
		modelElement.setComment(opr.getComment(null));
		modelElement.setElement(opr.getLocalName());
		return modelElement;
	}

	/**
	 * Generates the predicate tree of the owl model starting with the given
	 * root predicate.
	 * 
	 * @param rootPredicate
	 *            the owl root object property from which to start constructing
	 *            the tree.
	 * @return a composite tree representation of {@link ITreeNode}.
	 * @throws OntElementNotFoundException
	 *             is thrown when the rootPredicate does not exist in the
	 *             ontology.
	 */
	public ITreeNode generatePredicateTree(String rootPredicate)
			throws OntElementNotFoundException {
		ObjectProperty objProp = this.ontmodel.getObjectProperty(rootPredicate);
		if (objProp == null) {
			throw new OntElementNotFoundException("The root object property "
					+ rootPredicate + " does not exist!");
		}
		ITreeNode rootElement = populateElement(objProp, null);
		generateChildConcepts(objProp, rootElement,
				new ArrayList<OntProperty>(), 0);
		return rootElement;
	}

	private void generateChildConcepts(OntProperty opr, ITreeNode parentModel,
			List<OntProperty> occurs, int depth) {

		ITreeNode curElement = populateElement(opr, parentModel);
		parentModel.addChild(curElement);

		// recurse to the next level down
		if (opr.canAs(ObjectProperty.class) && !occurs.contains(opr)) {
			for (ExtendedIterator it = opr.listSubProperties(true); it
					.hasNext();) {
				OntProperty subopr = (OntProperty) it.next();
				// we push this expression on the occurs list before we recurse
				occurs.add(opr);
				generateChildConcepts(subopr, curElement, occurs, depth + 1);
				occurs.remove(opr);
			}
		}

	}

}
