package hok.ui.wizard;

import hok.model.ITreeNode;
import hok.model.TreeElement;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;

/**
 * The ConstraintDialog is used to select the predicates or - depending on the
 * type of the dialog - concepts the user can choose from to constrain the
 * design problem in the wizard.
 * 
 * @author holger
 * 
 */
public class ConstraintDialog extends TitleAreaDialog {

	private static final String CONCEPT_IMAGE = "OWLIndividual";

	private static final String PREDICATE_IMAGE = "OWLObjectProperty";

	/**
	 * The type of the dialog. Because both the predicates and the concepts are
	 * represented by the same model and can be displayed in a TreeViewer, the
	 * same dialog class can be used.
	 * 
	 */
	public static enum DialogType {
		PredicateType, ConceptType
	};

	private DialogType type;

	private ITreeNode treeNode;

	private TreeViewer treeViewer;

	private ITreeNode selectedElement;

	private ViewLabelProvider labelProvider;

	private static Object[] EMPTY_ARRAY = new Object[0];

	private static ImageRegistry imageRegistry = new ImageRegistry();

	/**
	 * Note: An image registry owns all of the image objects registered with it,
	 * and automatically disposes of them the SWT Display is disposed.
	 */
	static {
		String iconPath = "icons/";
		imageRegistry.put(CONCEPT_IMAGE, Activator.getImageDescriptor(iconPath
				+ CONCEPT_IMAGE + ".gif"));
		imageRegistry.put(PREDICATE_IMAGE, Activator
				.getImageDescriptor(iconPath + PREDICATE_IMAGE + ".gif"));
	}

	/**
	 * The constructor needs a parent shell and the Dialogtype (either
	 * ConceptType or PredicateType). The DialogType decides which icon is
	 * displayed for the tree elements and the content of the title and message
	 * bar
	 * 
	 * @param parentShell
	 *            the parent shell to set.
	 * @param type
	 *            the dialog type to set.
	 */
	protected ConstraintDialog(Shell parentShell, DialogType type) {
		super(parentShell);
		this.type = type;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		Control contents = super.createContents(parent);

		switch (getType()) {
		case ConceptType:
			setTitle("Concept Constraint");
			// Set the message
			setMessage("Choose the constraining concept from the list!",
					IMessageProvider.NONE);
			break;
		case PredicateType:
			setTitle("Predicate Constraint");
			// Set the message
			setMessage("Choose the constraining predicate from the list!",
					IMessageProvider.NONE);
			break;
		}

		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite comp = (Composite) super.createDialogArea(parent);

		// Create the tree viewer as a child of the composite parent
		treeViewer = new TreeViewer(comp);

		treeViewer.setAutoExpandLevel(2);

		treeViewer.setContentProvider(new ViewContentProvider());
		labelProvider = new ViewLabelProvider();
		treeViewer.setLabelProvider(labelProvider);

		treeViewer.setUseHashlookup(true);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.verticalSpan = 1;
		gridData.horizontalSpan = 1;
		treeViewer.getTree().setLayoutData(gridData);

		treeViewer.getTree().setLinesVisible(true);

		if (getTreeModel() == null) {
			treeViewer.setInput(getInitalInput());
		} else {
			treeViewer.setInput(getTreeModel());
		}

		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent event) {
				if (event.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection selection = (IStructuredSelection) event
							.getSelection();
					Object firstElement = selection.getFirstElement();
					if (firstElement instanceof ITreeNode) {
						selectedElement = (ITreeNode) firstElement;
						getButton(IDialogConstants.OK_ID).setEnabled(true);
					}
				}

			}

		});
		
//		 Implement a "fake" tooltip
		final Listener labelListener = new Listener () {
			public void handleEvent (Event event) {
				Label label = (Label)event.widget;
				Shell shell = label.getShell ();
				switch (event.type) {
					case SWT.MouseDown:
						Event e = new Event ();
						e.item = (TreeItem) label.getData ("_TABLEITEM");
						// Assuming table is single select, set the selection as if
						// the mouse down event went through to the table
						treeViewer.getTree().setSelection (new TreeItem [] {(TreeItem) e.item});
						treeViewer.getTree().notifyListeners (SWT.Selection, e);
						shell.dispose ();
						treeViewer.getTree().setFocus();
						break;
					case SWT.MouseExit:
						shell.dispose ();
						break;
				}
			}
		};
		
		Listener tableListener = new Listener () {
			Shell tip = null;
			Label label = null;
			public void handleEvent (Event event) {
				switch (event.type) {
					case SWT.Dispose:
					case SWT.KeyDown:
					case SWT.MouseMove: {
						if (tip == null) break;
						tip.dispose ();
						tip = null;
						label = null;
						break;
					}
					case SWT.MouseHover: {
						TreeItem item = treeViewer.getTree().getItem (new Point (event.x, event.y));
						
						if (item != null) {
							TreeElement elem = (TreeElement) item.getData();
							if (tip != null  && !tip.isDisposed ()) tip.dispose ();
							tip = new Shell ( SWT.ON_TOP | SWT.NO_FOCUS | SWT.TOOL);
							//tip.setBackground (SWT.COLOR_INFO_BACKGROUND);
							FillLayout layout = new FillLayout ();
							layout.marginWidth = 2;
							tip.setLayout (layout);
							label = new Label (tip, SWT.NONE);
//							label.setForeground (getSystemColor (SWT.COLOR_INFO_FOREGROUND));
//							label.setBackground (display.getSystemColor (SWT.COLOR_INFO_BACKGROUND));
							label.setData ("_TABLEITEM", item);
							label.setText (elem.getComment());
							label.addListener (SWT.MouseExit, labelListener);
							label.addListener (SWT.MouseDown, labelListener);
							Point size = tip.computeSize (SWT.DEFAULT, SWT.DEFAULT);
							Rectangle rect = item.getBounds (0);
							Point pt = treeViewer.getTree().toDisplay (rect.x, rect.y);
							
							tip.setBounds (pt.x+10, pt.y+10, size.x, size.y);
							tip.setVisible (true);
						}
					}
				}
			}
		};
		
		treeViewer.getTree().addListener (SWT.Dispose, tableListener);
		treeViewer.getTree().addListener (SWT.KeyDown, tableListener);
		treeViewer.getTree().addListener (SWT.MouseMove, tableListener);
		treeViewer.getTree().addListener (SWT.MouseHover, tableListener);
		
		return comp;
	}

	private ITreeNode getInitalInput() {
		TreeElement m = new TreeElement(null);
		m.setElement("root");
		m.setLabel("root");
		TreeElement e = new TreeElement(m);
		e.setElement("element");
		e.setLabel("element");
		m.addChild(e);
		m.addChild(e);
		m.addChild(e);
		TreeElement ee = new TreeElement(e);
		ee.setElement("ee");
		ee.setLabel("ee");
		e.addChild(ee);
		return m;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);

		getButton(IDialogConstants.OK_ID).setEnabled(false);

		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, true);
	}

	/**
	 * The content provider mediates between the TreeViewer and the ATreeModel.
	 * 
	 * @author holger
	 * 
	 */
	class ViewContentProvider implements ITreeContentProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
		 */
		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof ITreeNode) {
				ITreeNode m = (ITreeNode) parentElement;
				return m.getChildren(m).toArray();
			}
			return EMPTY_ARRAY;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
		 */
		public Object getParent(Object element) {
			if (element instanceof ITreeNode) {
				return ((ITreeNode) element).getParent();
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
		 */
		public boolean hasChildren(Object element) {
			if (element instanceof ITreeNode) {
				return ((ITreeNode) element).hasChildren();
			}
			return false;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

			if (oldInput != null) {
				removeListenerFrom((ITreeNode) oldInput);
			}
			if (newInput != null) {
				addListenerTo((ITreeNode) newInput);
			}

		}

		private void addListenerTo(ITreeNode model) {
			// TODO Auto-generated method stub

		}

		private void removeListenerFrom(ITreeNode model) {
			// TODO Auto-generated method stub

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
			// TODO Auto-generated method stub

		}

	}

	/**
	 * The label provider provides the text and the icon for the tree elements.
	 * 
	 * @author holger
	 * 
	 */
	class ViewLabelProvider extends LabelProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
		 */
		@Override
		public Image getImage(Object element) {
			switch (getType()) {
			case ConceptType:
				return imageRegistry.get(CONCEPT_IMAGE);
			case PredicateType:
				return imageRegistry.get(PREDICATE_IMAGE);
			default:
				return null;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
		 */
		@Override
		public String getText(Object element) {
			if (element instanceof TreeElement) {
				return ((TreeElement) element).getElement();
			}
			return super.getText(element);
		}

	}

	/**
	 * Returns the type of the Dialog
	 * 
	 * @return either ConceptType or PredicateType.
	 */
	public DialogType getType() {
		return type;
	}

	/**
	 * Sets the type of the Dialog.
	 * 
	 * @param type
	 *            either ConceptType or PredicateType.
	 */
	public void setType(DialogType type) {
		this.type = type;
	}

	/**
	 * Returns the tree model.
	 * 
	 * @return the tree model.
	 */
	public ITreeNode getTreeModel() {
		return treeNode;
	}

	/**
	 * Sets the tree model. This method has to be called before open().
	 * 
	 * @param treeNode
	 *            the tree model to set.
	 */
	public void setTreeModel(ITreeNode treeNode) {
		this.treeNode = treeNode;
	}

	/**
	 * Returns the element that is selected by the user.
	 * 
	 * @return the selected tree element or null.
	 */
	public ITreeNode getSelectedElement() {
		return selectedElement;
	}

}
