package hok.ui.wizard;

import hok.model.PatternElement;
import hok.model.PatternModel;

import java.util.Arrays;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

/**
 * The ResultDialog is a TitleAreaDialog with a TableViewer. It presents the
 * suitable design patterns matching the design problem configured in the
 * wizard. Befor calling open() it is needed to set the pattern model via
 * setPatternModel(model).
 * 
 * @author holger
 * 
 */
public class ResultDialog extends TitleAreaDialog {

	private static final String[] COLUMN_NAMES = new String[] {
			"Design Pattern", "Description" };

	private static final int[] COLUMN_ALIGNMENT = new int[] { SWT.LEFT,
			SWT.LEFT };

	private static final int[] COLUMN_WIDTH = new int[] { 150, 500 };

	private static final String PATTERN_ICON = "sunny.gif";

	private static ImageRegistry imageRegistry = new ImageRegistry();

	private TableViewer viewer;

	private PatternModel patternModel;

	/**
	 * Note: An image registry owns all of the image objects registered with it,
	 * and automatically disposes of them the SWT Display is disposed.
	 */
	static {
		String iconPath = "icons/";
		imageRegistry.put(PATTERN_ICON, Activator.getImageDescriptor(iconPath
				+ PATTERN_ICON));
	}

	/**
	 * @param parentShell
	 */
	public ResultDialog(Shell parentShell) {
		super(parentShell);
		// TODO Auto-generated constructor stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		Control contents = super.createContents(parent);

		setTitle("Suitable Design Pattern(s)");

		setMessage(
				"The following Design Patterns solve the given problem description!",
				IMessageProvider.NONE);

		return contents;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite contents = (Composite) super.createDialogArea(parent);
		createTableViewer(contents);
		return contents;

	}

	/**
	 * @param patternModel
	 */
	public void setPatternModel(PatternModel patternModel) {
		this.patternModel = patternModel;
	}

	private void createTableViewer(Composite contents) {
		viewer = new TableViewer(contents, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.verticalSpan = 1;
		gridData.horizontalSpan = 1;
		viewer.getTable().setLayoutData(gridData);

		viewer.getTable().setLinesVisible(true);
		viewer.setColumnProperties(getColumnNames());
		createColumns(viewer.getTable());
		viewer.getTable().setHeaderVisible(true);

		viewer.setContentProvider(new ViewContentProvider());

		viewer.setInput(this.patternModel);

		ViewLabelProvider labelProvider = new ViewLabelProvider();
		viewer.setLabelProvider(labelProvider);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CLOSE_LABEL, true);
	}

	private String[] getColumnNames() {
		return COLUMN_NAMES;
	}

	/**
	 * Returns the names of the table columns as a list.
	 * 
	 * @return a list of column names
	 */
	public java.util.List getColumnNamesAsList() {
		return Arrays.asList(COLUMN_NAMES);
	}

	private int getColumnCount() {
		return COLUMN_NAMES.length;
	}

	private int[] getColumnAlignment() {
		return COLUMN_ALIGNMENT;
	}

	private int[] getColumnWidth() {
		return COLUMN_WIDTH;
	}

	private void createColumns(Table table) {
		for (int i = 0; i < getColumnCount(); i++) {
			TableColumn column = new TableColumn(table,
					getColumnAlignment()[i], i);
			column.setText(getColumnNames()[i]);
			column.setWidth(getColumnWidth()[i]);
		}
	}

	/**
	 * The content provider mediates between the view and the pattern model.
	 * 
	 * @author holger
	 * 
	 */
	class ViewContentProvider implements IStructuredContentProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {

			return patternModel.getPatterns().toArray();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
			// nothing to dispose

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			// nothing to change

		}
	}

	/**
	 * Provides the text and the icon of the table elements.
	 * 
	 * @author holger
	 * 
	 */
	class ViewLabelProvider extends LabelProvider implements
			ITableLabelProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return (columnIndex == 0) ? imageRegistry.get(PATTERN_ICON) : null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object obj, int columnIndex) {
			String result = "";
			PatternElement element = (PatternElement) obj;
			switch (columnIndex) {
			case 0:
				result = element.getName();
				break;
			case 1:
				result = element.getDescription();
				break;
			default:
				break;
			}
			return result;
		}
	}
}
